<?php // $Id: lib-install-db.inc.php,v 1.17 2002/05/03 14:17:36 ciaccia Exp $

/************************************************************************/
/* phpPgAds                                                             */
/* ========                                                             */
/*                                                                      */
/* Copyright (c) 2001 by the phpPgAds developers                        */
/* http://sourceforge.net/projects/phppgads                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/************************************************************************/



// Define
define ('phpAds_databaseUpgradeSupported', true);
define ('phpAds_databaseCreateSupported', true);
define ('phpAds_tableTypesSupported', false);



/*********************************************************/
/* Create the database                                   */
/*********************************************************/

function phpAds_checkDatabase ()
{
	// Get the database structure
	$dbstructure	 = phpAds_prepareDatabaseStructure();
	$availabletables = array();
	
	// Get table names
	$res = phpAds_dbQuery(phpAds_pgShowTables);
	while ($row = phpAds_dbFetchRow($res))
		$availabletables[] = $row[0];
	
	$result = false;
	
	for (reset($dbstructure);
		$key = key($dbstructure);
		next($dbstructure))
	{
		if (is_array($availabletables) && in_array ($key, $availabletables))
		{
			// Table exists
			$result = true;
		}
	}
	
	return $result;
}



/*********************************************************/
/* Upgrade the database to the latest structure          */
/*********************************************************/

function phpAds_upgradeDatabase ($tabletype = '')
{	
	// Prepare data for banner table splitting
	phpAds_upgradeSplitBannersPrepare();

	// Call create function to get the upgrade
	phpAds_createDatabase($tabletype, true);
	
	// Split banners into two tables and
	// generate banner html cache
	phpAds_upgradeSplitBanners();
	
	// Detect version of needed plugins
	phpAds_upgradeDetectPluginVersion();
	
	// Update banner cache off all banners
	phpAds_upgradeHTMLCache();

	return true;
}



/*********************************************************/
/* Create the database                                   */
/*********************************************************/

function phpAds_createDatabase ($tabletype = '', $upgrade = false)
{
	global $phpAds_pgVersion;

	// Get the database structure
	$dbstructure	 = phpAds_prepareDatabaseStructure();
	$availabletables = array();
	
	// Get table names
	$res = phpAds_dbQuery(phpAds_pgShowTables);
	while ($row = phpAds_dbFetchRow($res))
		$availabletables[] = $row[0];

	// For safety reasons upgrade in three steps
	
	phpAds_dbQuery("BEGIN");

	// If upgrading backup existing tables
	for (reset($dbstructure);
		$upgrade && ($key = key($dbstructure));
		next($dbstructure))
	{
		if (is_array($availabletables) && in_array ($key, $availabletables))
		{
			// Table exists, backup
			phpAds_backupTable ($key);
		}
	}

	phpAds_dbQuery("COMMIT");
	phpAds_dbQuery("BEGIN");

	// Drop existing tables
	for (reset($dbstructure);
		$key = key($dbstructure);
		next($dbstructure))
	{
		if (is_array($availabletables) && in_array ($key, $availabletables))
		{
			// Table exists, drop it
			phpAds_dropTable ($key);
		}
	}

	phpAds_dbQuery("COMMIT");

	// Create database structure
	for (reset($dbstructure);
		$key = key($dbstructure);
		next($dbstructure))
	{
		// Table doesn't exists, create
		phpAds_createTable ($key, $dbstructure[$key], $tabletype);
	}
	
    if ($upgrade)
    {
	    // Turn off autocommit to improve performance
	    phpAds_dbQuery("BEGIN");
	    // Turn off DEFERRABLE constraints to avoid referencial integrity violations
	    phpAds_dbQuery("SET CONSTRAINTS ALL DEFERRED") or phpAds_sqlDie();


	    // If upgrading restore data
	    for (reset($dbstructure);
		    $key = key($dbstructure);
		    next($dbstructure))
	    {
		    if (is_array($availabletables) && in_array ($key, $availabletables))
		    {
			    // Table exists, restore
			    phpAds_restoreTable ($key, $dbstructure[$key]);
		    }
	    }

    	// If upgrading drop temp tables
    	for (reset($dbstructure);
	    	 $key = key($dbstructure);
		    next($dbstructure))
	    {
		    if (is_array($availabletables) && in_array ($key, $availabletables))
		    {
			    // Table exists, backup
			    $tmp_name = substr('tmp_'.$key, 0, 31);
			    phpAds_dropTable ($tmp_name);
		    }
	    }
 
        // Commit changes
	    phpAds_dbQuery("COMMIT") or phpAds_sqlDie();
    }

	// Drop and create SQL functions or recreate using CREATE OR REPLACE FUNCTION if available
	if ($phpAds_pgVersion < 72000)
		$sqlfunctions = phpAds_readDatabaseStructure('functions_drop.sql');
	else
		$sqlfunctions = array();

	$sqlfunctions = array_merge($sqlfunctions, phpAds_readDatabaseStructure('functions.sql'));
	while (list(, $v) = each($sqlfunctions))
	{
		if ($phpAds_pgVersion >= 72000 && ereg('CREATE FUNCTION ', $v))
			$v = str_replace('CREATE FUNCTION', 'CREATE OR REPLACE FUNCTION', $v);

		phpAds_dbQuery($v);
	}

	return true;
}




/*********************************************************/
/* Backup a table                                        */
/*********************************************************/

function phpAds_backupTable ($name)
{
	$tmp_name = substr('tmp_'.$name, 0, 31);
	phpAds_dbQuery("SELECT * INTO $tmp_name FROM $name") or phpAds_sqlDie();

	return true;
}



/*********************************************************/
/* Restore a table                                       */
/*********************************************************/

function phpAds_restoreTable ($name, $structure)
{
	global $phpAds_config;
	
	$tmp_name = substr('tmp_'.$name, 0, 31);

	$src = $tmp_name;
	$dst = $name;
	
	// If source table is empty do nothing
	if (!phpAds_dbResult(phpAds_dbQuery("SELECT COUNT(*) FROM $src"), 0, 0))
		return true;
	
	$rsrc = phpAds_dbQuery("SELECT * FROM $src WHERE 1 = 0") or phpAds_sqlDie();
	$rdst = phpAds_dbQuery("SELECT * FROM $dst WHERE 1 = 0") or phpAds_sqlDie();
	
	for ($x = 0; $x < pg_numfields($rsrc); $x++)
		$fsrc[pg_fieldname($rsrc, $x)] = pg_fieldtype($rsrc, $x);
		
	for ($x = 0; $x < pg_numfields($rdst); $x++)
		$fdst[pg_fieldname($rdst, $x)] = pg_fieldtype($rdst, $x);
		
	$fields = array();
	$values = array();
	
	while (list($k, $v) = each($fdst))
	{
		if (isset($fsrc[$k]))
		{
			// Pg cannot cast int2 to int8 directly
			if ($fsrc[$k] == 'int2' && $v == 'int8')
				$v =  'int4';

			$fields[] = $k;
			
			if ($dst == $phpAds_config['tbl_banners'] &&
				($k == 'status' || $k == 'autohtml'))
				$values[] = "CASE WHEN $k IS NULL THEN ".($v == 'bool' ? "'f'" : "''")." ELSE $k::$v END";
			elseif ($dst == $phpAds_config['tbl_acls'] &&
				$k == 'acl_ad' && $fsrc[$k] == 'bool')
				$values[] = "CASE WHEN $k THEN 'allow'::$v ELSE 'deny'::$v END";
			elseif ($dst == $phpAds_config['tbl_zones'] &&
				$k == 'cachetimestamp' && $fsrc[$k] == 'timestamp')
				$values[] = "0";
			elseif ($dst == $phpAds_config['tbl_zones'] &&
				$k == 'cachecontents' && $fsrc['cachetimestamp'] == 'timestamp')
				$values[] = "NULL";
			elseif ($dst == $phpAds_config['tbl_clients'] && $k == 'parent')
				$values[] = "CASE WHEN $k = 0 THEN null ELSE $k::$v END";
			else
				$values[] = "$k::$v";
		}
	}
	
	phpAds_dbQuery("INSERT INTO $dst (".join(', ', $fields).") SELECT ".
		join(', ', $values)." FROM $src") or phpAds_sqlDie();

	if (isset($structure['sequence']))
	{
		for (reset($structure['sequence']); list($k, $v) = each($structure['sequence']); )
			phpAds_dbQuery("SELECT setval('$k', (SELECT CASE WHEN MAX($v) > 0 THEN MAX($v) ELSE NULL END FROM $dst))") or phpAds_sqlDie();
	}

	return true;
}



/*********************************************************/
/* Create a table                                        */
/*********************************************************/

function phpAds_createTable ($name, $structure, $tabletype = '')
{
	$columns = $structure['columns'];
	if (isset($structure['primary'])) $primary = $structure['primary'];
	if (isset($structure['index']))   $index   = $structure['index'];
	if (isset($structure['unique']))  $unique  = $structure['unique'];
	if (isset($structure['foreign'])) $foreign = $structure['foreign'];
	if (isset($structure['sequence'])) $sequence = $structure['sequence'];
	
	// Create empty array
	$createdefinitions = array();
	
	// Get sequences names
	$availablesequences = array();
	$res = phpAds_dbQuery(phpAds_pgShowSequences);
	while ($row = phpAds_dbFetchRow($res))
		$availablesequences[] = $row[0];

	// Add sequences
	if (isset($sequence) && is_array($sequence) && sizeof($sequence) > 0)
	{
		for (reset($sequence); $key = key($sequence); next($sequence))
		{
			if (in_array($key, $availablesequences))
				phpAds_dbQuery("SELECT setval('".$key."', NULL)");
			else
				phpAds_dbQuery("CREATE SEQUENCE ".$key) or phpAds_sqlDie();
		}
	}

	// Add columns
	for (reset($columns); $key = key($columns);	next($columns))
		$createdefinitions[] = $key." ".$columns[$key];
	
	if (isset($primary) && is_array($primary) && sizeof($primary) > 0)
		$createdefinitions[] = "CONSTRAINT ".$name."_pkey PRIMARY KEY (".implode(",", $primary).")";

	if (isset($foreign) && is_array($foreign) && sizeof($foreign) > 0)
	{
		for (reset($foreign);$key=key($foreign);next($foreign))
			$createdefinitions[] = "CONSTRAINT $key ".
				"FOREIGN KEY (".implode("_", $foreign[$key]['keys']).") ".
				"REFERENCES ".$foreign[$key]['references']." (".implode(",", $foreign[$key]['refkeys']).") ".
				$foreign[$key]['extra'];
	}	

	if (is_array($createdefinitions) &&
		sizeof($createdefinitions) > 0)
	{
		$query  = "CREATE TABLE $name (";
		$query .= implode (", ", $createdefinitions);
		$query .= ")";
		
		// Tabletype
		if ($tabletype != '')
			$query .= " TYPE=".$tabletype;
			
		phpAds_dbQuery($query) or phpAds_sqlDie();
	}

	// Add indexes
	if (isset($index) && is_array($index) && sizeof($index) > 0)
	{
		for (reset($index);$key=key($index);next($index))
			phpAds_dbQuery("CREATE INDEX $key ON ".$name." ".$index[$key]['using']." (".implode(",", $index[$key]['columns']).")") or phpAds_sqlDie();
	}

	// Add unique indexes
	if (isset($unique) && is_array($unique) && sizeof($unique) > 0)
	{
		for (reset($unique);$key=key($unique);next($unique))
			$createdefinitions[] = "CREATE UNIQUE INDEX $key ON ".$name." (".implode(",", $unique[$key]['columns']).")" or phpAds_sqlDie();
	}
}



/*********************************************************/
/* Drop an existing table                                */
/*********************************************************/

function phpAds_dropTable ($name)
{
	return phpAds_dbQuery("DROP TABLE ".$name);
}



/*********************************************************/
/* Read the database structure from a sql file           */
/*********************************************************/

function phpAds_readDatabaseStructure ($filename = 'all.sql')
{
	global $phpAds_config;
	
	$sql = join("", file(phpAds_path."/misc/".$filename));
	
	// Stripping comments
	$sql = ereg_replace("-- [^\n]*\n", "\n", $sql);
	$sql = ereg_replace("$#[^\n]*\n", "\n", $sql);
	
	// Stripping (CR)LFs
	//$sql = str_replace("\r?\n\r?", "", $sql);
	$sql = str_replace("\r", " ", $sql);
	$sql = str_replace("\n", " ", $sql);
	
	
	// Unifying duplicate blanks
	$sql = ereg_replace("[[:blank:]]+", " ", $sql);
	
	$sql = explode(";", $sql);
	
	// Replacing table names to match config.inc.php
	for ($i=0;$i<sizeof($sql);$i++)
	{
		if (ereg ("CREATE TABLE (phpads_[^ ]*) \(", $sql[$i], $regs))
		{
			$tablename = str_replace ("phpads_", "tbl_", $regs[1]);
			
			if (isset($phpAds_config[$tablename]))
				$sql[$i] = str_replace ($regs[1], $phpAds_config[$tablename], $sql[$i]);
		}
		
		if (ereg ("CREATE INDEX [A-Za-z0-9_]+ ON (phpads_[^ ]+)", $sql[$i], $regs))
		{
			$tablename = str_replace ("phpads_", "tbl_", $regs[1]);
			
			if (isset($phpAds_config[$tablename]))
				$sql[$i] = str_replace ($regs[1], $phpAds_config[$tablename], $sql[$i]);
		}

		if (ereg ("REFERENCES (phpads_[^ ]+)", $sql[$i], $regs))
		{
			$tablename = str_replace ("phpads_", "tbl_", $regs[1]);
			
			if (isset($phpAds_config[$tablename]))
				$sql[$i] = str_replace ('REFERENCES '.$regs[1], 'REFERENCES '.$phpAds_config[$tablename], $sql[$i]);
		}
	}
	
	// Create an array with an element for each query
	return $sql;
}



/*********************************************************/
/* Parse the an sql file and return all queries          */
/*********************************************************/

function phpAds_prepareDatabaseStructure()
{
	global $phpAds_config;
	global $phpAds_pgVersion;
	
	// Get Postgres version
	if (ereg("^PostgreSQL ([0-9]+)\.([0-9]+)(\.[0-9]+)?",
		phpAds_dbResult(phpAds_dbQuery('SELECT version()'), 0, 0), $regs))
	{
		$phpAds_pgVersion = $regs[1]*10000+$regs[2]*100+$regs[3]*1;
	}
	else
		$phpAds_pgVersion = 0;

	$dbstructure = array();

	// Read the all.sql file
	$queries = phpAds_readDatabaseStructure ();
	
	
	for ($i=0;$i<sizeof($queries)-1;$i++)
	{
		if (ereg ("CREATE TABLE ([^\(]*) \((.*)\)", $queries[$i], $regs))
		{
			$tablename   = $regs[1];
			$definitions = $regs[2];
			
			$definitions = explode (", ", $definitions);
			
			for ($j=0;$j<sizeof($definitions);$j++)
			{
				$definition = trim($definitions[$j]);
				
				if (ereg("^PRIMARY KEY \((.*)\)$", $definition, $regs))
				{
					$items = explode(",", $regs[1]);
					for ($k=0;$k<sizeof($items);$k++)
						$dbstructure[$tablename]['primary'][] = $items[$k];
				}
				elseif (ereg("^UNIQUE \((.*)\)$", $definition, $regs))
				{
					$items = explode(",", $regs[1]);
					$idxname = substr($tablename.'_'.join('_', $items).'_udx', 0, 31);
					for ($k=0;$k<sizeof($items);$k++)
						$dbstructure[$tablename]['unique'][$idxname]['columns'][] = $items[$k];
				}
				elseif (ereg("^FOREIGN KEY \(([^ ]*)\) REFERENCES ([^ ]*) \(([^)]*)\) (.*)$", $definition, $regs))
				{
					// Add foreign keys and index
					$items = explode(",", $regs[1]);
					$idxname = substr($tablename.'_'.join('_', $items), 0, 28);
					for ($k=0;$k<sizeof($items);$k++)
						$dbstructure[$tablename]['foreign'][$idxname.'_fk']['keys'][] = $items[$k];

					// Add referenced table
					$dbstructure[$tablename]['foreign'][$idxname.'_fk']['references'] = $regs[2];

					// Add referenced table fields
					$items = explode(",", $regs[3]);
					for ($k=0;$k<sizeof($items);$k++)
						$dbstructure[$tablename]['foreign'][$idxname.'_fk']['refkeys'][] = $items[$k];

					// Add extra
					$dbstructure[$tablename]['foreign'][$idxname.'_fk']['extra'] = $regs[4];
				}
				elseif (ereg("^([^ ]*) (.*)$", $definition, $regs))
				{
					if (ereg("^serial ", $regs[2]))
					{
						$seqname = substr($tablename.'_'.$regs[1].'_seq', 0, 31);
						$regs[2] = "int4 NOT NULL DEFAULT nextval('".$seqname."')";
						$dbstructure[$tablename]['sequence'][$seqname] = $regs[1];
					}
					elseif ($phpAds_pgVersion >= 70200 && ereg("^timestamp ", $regs[2]))
						$regs[2] = str_replace('timestamp', 'timestamp(0)', $regs[2]);

					$dbstructure[$tablename]['columns'][$regs[1]] = $regs[2];
				}
			}
		}
		elseif (ereg ("CREATE( UNIQUE)? INDEX ([^ ]*) ON ([^ ]+) (USING [A-Zaz-]+ )?\((.*)\)", $queries[$i], $regs))
		{
			$index = empty($regs[1]) ? 'index' : 'unique';
			$idxname = substr($regs[2], 0, 31);
			$dbstructure[$regs[3]][$index][$idxname]['using'] = $regs[4];
			$items = explode(",", $regs[5]);
			for ($k=0;$k<sizeof($items);$k++)
				$dbstructure[$regs[3]][$index][$idxname]['columns'][] = $items[$k];
		}
	}
	
	return $dbstructure;
}


/*********************************************************/
/* Version specific updates                              */
/*********************************************************/

function phpAds_upgradeSplitBannersPrepare ()
{
	global $phpAds_config;

	// Check if splitting is needed
	if (!isset($phpAds_config['config_version']) ||	$phpAds_config['config_version'] < 200.041)
	{
		// Backup needed data into a temporary table
		phpAds_dbQuery ("DROP TABLE phppgads_splitbanners");
		phpAds_dbQuery ("SELECT bannerid, banner, format INTO phppgads_splitbanners FROM ".$phpAds_config['tbl_banners']) or phpAds_sqlDie();
	}
}

function phpAds_upgradeSplitBanners ()
{
	global $phpAds_config;
	
	// Check if splitting is needed
	if (!isset($phpAds_config['config_version']) ||	$phpAds_config['config_version'] < 200.041)
	{
		// Fetch all banners
		$res = phpAds_dbQuery ("SELECT * FROM ".$phpAds_config['tbl_banners']." JOIN phppgads_splitbanners USING (bannerid)");
		
		while ($row = phpAds_dbFetchArray($res))
		{
			$banners[] = $row;
		}

		for ($i=0; $i < count($banners); $i++)
		{
			// Requote fields
			$banners[$i]['alt'] 		= phpAds_htmlQuotes(stripslashes($banners[$i]['alt']));
			$banners[$i]['bannertext'] 	= phpAds_htmlQuotes(stripslashes($banners[$i]['bannertext']));
			
			// Resplit keywords
			if (isset($banners[$i]['keyword']) && $banners[$i]['keyword'] != '')
			{
				$keywordArray = split('[ ,]+', trim($banners[$i]['keyword']));
				$banners[$i]['keyword'] = implode(' ', $keywordArray);
			}
			
			// Determine storagetype
			switch ($banners[$i]['format'])
			{
				case 'url':		$banners[$i]['storagetype'] = 'url';	break;
				case 'html':	$banners[$i]['storagetype'] = 'html';	break;
				case 'web':		$banners[$i]['storagetype'] = 'web';	break;
				default:		$banners[$i]['storagetype'] = 'sql';	break;
			}
			
			switch ($banners[$i]['storagetype'])
			{
				case 'sql':
					
					// Determine contenttype
					$banners[$i]['contenttype']  = $banners[$i]['format'];
					
					// Load and delete old Postgres large object
					if (ereg("^oid:([0-9]+)$", $banners[$i]['banner'], $match))
					{
						$banners[$i]['banner'] = phpPgAds_lo_read($match[1]);
						phpAds_dbQuery("BEGIN");
						@pg_lounlink($GLOBALS['phpAds_db_link'], $match[1]);
						phpAds_dbQuery("COMMIT");
					}

					// Store the file
					$banners[$i]['filename']	 = 'banner_'.$banners[$i]['bannerid'].'.'.$banners[$i]['contenttype'];
					$banners[$i]['filename'] 	 = phpAds_ImageStore($banners[$i]['storagetype'], $banners[$i]['filename'], $banners[$i]['banner']);
					$banners[$i]['imageurl']	 = $phpAds_config['url_prefix'].'/adimage.php?filename='.$banners[$i]['filename']."&contenttype=".$banners[$i]['contenttype'];

					$banners[$i]['htmltemplate'] = phpAds_getBannerTemplate($banners[$i]['contenttype']);
					$banners[$i]['htmlcache']    = addslashes(phpAds_getBannerCache($banners[$i]));
					$banners[$i]['htmltemplate'] = addslashes($banners[$i]['htmltemplate']);
					
					$banners[$i]['banner']		 = '';
					break;
				
				case 'web':
					// Get the contenttype
					$ext = substr($banners[$i]['banner'], strrpos($banners[$i]['banner'], ".") + 1);
					switch (strtolower($ext)) 
					{
						case 'jpeg': $banners[$i]['contenttype'] = 'jpeg';  break;
						case 'jpg':	 $banners[$i]['contenttype'] = 'jpeg';  break;
						case 'html': $banners[$i]['contenttype'] = 'html';  break;
						case 'png':  $banners[$i]['contenttype'] = 'png';   break;
						case 'gif':  $banners[$i]['contenttype'] = 'gif';   break;
						case 'swf':  $banners[$i]['contenttype'] = 'swf';   break;
					}
					
					// Store the file
					$banners[$i]['filename']	 = basename($banners[$i]['banner']);
					$banners[$i]['imageurl']	 = $banners[$i]['banner'];
					
					$banners[$i]['htmltemplate'] = phpAds_getBannerTemplate($banners[$i]['contenttype']);
					$banners[$i]['htmlcache']    = addslashes(phpAds_getBannerCache($banners[$i]));
					$banners[$i]['htmltemplate'] = addslashes($banners[$i]['htmltemplate']);

					$banners[$i]['banner']		 = '';
					break;
				
				case 'url':
					// Get the contenttype
					$ext = parse_url($banners[$i]['banner']);
					$ext = $ext['path'];
					$ext = substr($ext, strrpos($ext, ".") + 1);
					switch (strtolower($ext)) 
					{
						case 'jpeg': $banners[$i]['contenttype'] = 'jpeg';  break;
						case 'jpg':	 $banners[$i]['contenttype'] = 'jpeg';  break;
						case 'html': $banners[$i]['contenttype'] = 'html';  break;
						case 'png':  $banners[$i]['contenttype'] = 'png';   break;
						case 'gif':  $banners[$i]['contenttype'] = 'gif';   break;
						case 'swf':  $banners[$i]['contenttype'] = 'swf';   break;
					}
					
					$banners[$i]['imageurl']	 = $banners[$i]['banner'];
					
					$banners[$i]['htmltemplate'] = phpAds_getBannerTemplate($banners[$i]['contenttype']);
					$banners[$i]['htmlcache']    = addslashes(phpAds_getBannerCache($banners[$i]));
					$banners[$i]['htmltemplate'] = addslashes($banners[$i]['htmltemplate']);

					$banners[$i]['filename']	 = '';
					$banners[$i]['banner']		 = '';
					break;
				
				case 'html':
					// Get the contenttype
					$banners[$i]['contenttype']  = 'html';
					
					$banners[$i]['htmltemplate'] = stripslashes($banners[$i]['banner']);
					$banners[$i]['htmlcache']    = addslashes(phpAds_getBannerCache($banners[$i]));
					$banners[$i]['htmltemplate'] = addslashes($banners[$i]['htmltemplate']);

					$banners[$i]['imageurl']	 = '';
					$banners[$i]['filename']	 = '';
					$banners[$i]['banner']		 = '';
					break;
			}
			
			// Update the banner
			$res = phpAds_dbQuery ("
				UPDATE
					".$phpAds_config['tbl_banners']."
				SET
					storagetype = '".$banners[$i]['storagetype']."',
					contenttype = '".$banners[$i]['contenttype']."',
					filename = '".$banners[$i]['filename']."',
					imageurl = '".$banners[$i]['imageurl']."',
					htmltemplate = '".$banners[$i]['htmltemplate']."',
					htmlcache = '".$banners[$i]['htmlcache']."',
					alt = '".$banners[$i]['alt']."',
					status = '".$banners[$i]['status']."',
					bannertext = '".$banners[$i]['bannertext']."',
					keyword = '".$banners[$i]['keyword']."'
				WHERE
					bannerid = ".$banners[$i]['bannerid']."
			") or phpAds_sqlDie();
		}
		
		// Drop temporary table
		$res = phpAds_dbQuery ("DROP TABLE phppgads_splitbanners");
	}
}

function phpAds_upgradeDetectPluginVersion ()
{
	global $phpAds_config;
	
	// Include swf library
	include ("lib-swf.inc.php");
	
	// Check if plugin detection is needed
	if (!isset($phpAds_config['config_version']) ||	$phpAds_config['config_version'] < 200.055)
	{
		// Fetch all banners
		$res = phpAds_dbQuery ("SELECT * FROM ".$phpAds_config['tbl_banners']);
		
		while ($row = phpAds_dbFetchArray($res))
			$banners[] = $row;
		
		for ($i=0; $i < count($banners); $i++)
		{
			if ($banners[$i]['storagetype'] == 'sql' ||
				$banners[$i]['storagetype'] == 'web')
			{
				$pluginversion = 0;
				$htmltemplate = $banners[$i]['htmltemplate'];
				
				
				if ($banners[$i]['contenttype'] == 'swf')
				{
					// Determine version
					$swf_file = phpAds_ImageRetrieve ($banners[$i]['storagetype'], $banners[$i]['filename']);
					$pluginversion = phpAds_SWFVersion($swf_file);
					
					// Update template
					$htmltemplate = ereg_replace ("#version=[^\']*'", "#version={pluginversion:4,0,0,0}'", $htmltemplate);
				}
				elseif ($banners[$i]['contenttype'] == 'dcr')
				{
					// Update template
					$htmltemplate = ereg_replace ("#version=[^\']*'", "#version={pluginversion:8,5,0,321}'", $htmltemplate);
				}
				
				
				$htmltemplate = addslashes ($htmltemplate);
				
				// Update the banner
				$res = phpAds_dbQuery ("
					UPDATE
						".$phpAds_config['tbl_banners']."
					SET
						pluginversion = '".$pluginversion."',
						htmltemplate = '".$htmltemplate."'
					WHERE
						bannerid = ".$banners[$i]['bannerid']."
				");
			}
		}
	}
}

function phpAds_upgradeHTMLCache ()
{
	global $phpAds_config;
	
	$res = phpAds_dbQuery("
		SELECT
			*
		FROM
			".$phpAds_config['tbl_banners']."
	");
	
	while ($current = phpAds_dbFetchArray($res))
	{
		// Rebuild filename
		if ($current['storagetype'] == 'sql')
			$current['imageurl'] = $phpAds_config['url_prefix']."/adimage.php?filename=".$current['filename']."&contenttype=".$current['contenttype'];
		
		if ($current['storagetype'] == 'web')
			$current['imageurl'] = $phpAds_config['type_web_url'].'/'.$current['filename'];
		
		// Rebuild cache
		$current['htmltemplate'] = stripslashes($current['htmltemplate']);
		$current['htmlcache']    = addslashes(phpAds_getBannerCache($current));
		
		phpAds_dbQuery("
			UPDATE
				".$phpAds_config['tbl_banners']."
			SET
				htmlcache = '".$current['htmlcache']."',
				imageurl  = '".$current['imageurl']."'
			WHERE
				bannerid = ".$current['bannerid']."
		");
	}
}



// PostgreSQL queries
define ('phpAds_pgShowTables', "
SELECT c.relname as \"name\", 'table'::text as \"type\", u.usename as \"owner\"
FROM pg_class c, pg_user u
WHERE c.relowner = u.usesysid AND c.relkind = 'r'
  AND c.relname !~ '^pg_'
UNION
SELECT c.relname as \"name\", 'table'::text as \"type\", NULL as \"owner\"
FROM pg_class c
WHERE c.relkind = 'r'
  AND not exists (select 1 from pg_user where usesysid = c.relowner)
  AND c.relname !~ '^pg_'
");

define ('phpAds_pgShowViews', "
SELECT c.relname as \"name\", 'view'::text as \"type\", u.usename as \"owner\"
FROM pg_class c, pg_user u
WHERE c.relowner = u.usesysid AND c.relkind = 'v'
  AND c.relname !~ '^pg_'
UNION
SELECT c.relname as \"name\", 'view'::text as \"type\", NULL as \"owner\"
FROM pg_class c
WHERE c.relkind = 'v'
  AND not exists (select 1 from pg_user where usesysid = c.relowner)
  AND c.relname !~ '^pg_'
");

define ('phpAds_pgShowSequences', "
SELECT c.relname as \"name\",
  (CASE WHEN relkind = 'S' THEN 'sequence'::text ELSE 'index'::text END) as \"type\",
  u.usename as \"owner\"
FROM pg_class c, pg_user u
WHERE c.relowner = u.usesysid AND relkind in ('S')
  AND c.relname !~ '^pg_'
UNION
SELECT c.relname as \"name\",
  (CASE WHEN relkind = 'S' THEN 'sequence'::text ELSE 'index'::text END) as \"type\",
  NULL as \"owner\"
FROM pg_class c
WHERE not exists (select 1 from pg_user where usesysid = c.relowner) AND relkind
 in ('S')
  AND c.relname !~ '^pg_'
");

?>